﻿using System;
using System.Collections.Specialized;
using System.IO.Compression;
using System.Web;

using Subtext.TestLibrary;
using Xunit;

namespace StackExchange.Profiling.Tests
{
    public class MiniProfilerHandlerTests
    {
        [Theory]
        [InlineData("BRILLANT", 404)]
        [InlineData("underscore.js", 404)]
        [InlineData("results-list", 200)]
        [InlineData("includes.min.js", 200)]
        [InlineData("includes.min.css", 200)]
        public void GivenContext_WhenAResourceIsRequested_ThenTheCorrectHttpStatusCodeIsReturned(string resourceName, int expectedHttpStatus)
        {
            var sut = new MiniProfilerHandler(new MiniProfilerOptions()
            {
                ResultsListAuthorize = null
            });

            var res = GetRequestResponseHttpStatus(sut, resourceName);
            Assert.Equal(expectedHttpStatus, res);
        }

        [Theory]
        [InlineData(true, 200)]
        [InlineData(false, 401)]
        public void GivenContext_WhenIndexIsRequested_ThenTheCorrectHttpStatusCodeIsReturned(bool isRequestAuthorized, int expectedHttpStatus)
        {
            var sut = new MiniProfilerHandler(new MiniProfilerOptions()
            {
                ResultsListAuthorize = _ => isRequestAuthorized
            });

            var res = GetRequestResponseHttpStatus(sut, "/results-index");
            Assert.Equal(expectedHttpStatus, res);
        }

		[Theory]
		[InlineData("gzip", typeof(GZipStream))]
		[InlineData("deflate", typeof(DeflateStream))]
		[InlineData("unknown", null)]
		[InlineData("", null)]
		public void GivenContext_WhenIndexIsRequested_ThenTheCorrectHttpStatusCodeIsReturnedType(string acceptEncoding, Type expectedEncodingFilterType)
		{
			// Arrange
			var sut = new MiniProfilerHandler(new MiniProfilerOptions());

			// Act
			var res = GetRequestResponseEncoding(sut, "includes.min.js", acceptEncoding);

			// Assert
			// due the limitations of the HttpSimulator, we can't access the header values because it needs iis integrated pipeline mode.
			// instead we return the type of the response filter

			if (expectedEncodingFilterType == null)
			{
				Assert.NotEqual(typeof(GZipStream), res);
				Assert.NotEqual(typeof(DeflateStream), res);
			}
			else
			{
				Assert.Equal(expectedEncodingFilterType, res);
			}
		}

        private static int GetRequestResponseHttpStatus(MiniProfilerHandler handler, string resourceName)
        {
            using (new HttpSimulator("/mini-profiler-resources/", @"c:\").SimulateRequest(new Uri("http://localhost/mini-profiler-resources"+resourceName)))
            {
                handler.ProcessRequest(HttpContext.Current);
                return HttpContext.Current.Response.StatusCode;
            }
        }

        private static Type GetRequestResponseEncoding(MiniProfilerHandler handler, string resourceName, string encoding)
        {
            var headers = new NameValueCollection
            {
                ["Accept-encoding"] = encoding
            };
            using (new HttpSimulator("/mini-profiler-resources/", @"c:\").SimulateRequest(new Uri("http://localhost/mini-profiler-resources"+resourceName), headers: headers))
            {
                handler.ProcessRequest(HttpContext.Current);

				////return HttpContext.Current.Response.Headers["Content-encoding"];
	            return HttpContext.Current.Response.Filter.GetType();
            }
        }
    }
}
